/*
 * Copyright (c) 1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: cbr.cc,v 1.4 96/10/31 12:36:19 mccanne Exp $ (LBL)";
#endif

#include "cbr.h"
#include "node.h"
#include "Tcl.h"
#include "packet.h"
#include "random.h"

static class CBRMatcher : public Matcher {
public:
	CBRMatcher() : Matcher("agent") {}
	TclObject* match(const char* id) {
		if (strcmp(id, "cbr") == 0)
			return (new CBR_Agent());
		return (0);
	}
} matcher_cbr;

CBR_Agent::CBR_Agent() : Agent(PT_CBR), random_(0)
{
	Tcl& tcl = Tcl::instance();
	link_time("ns_cbr", "interval", &interval_, 0);
	link_int("ns_cbr", "packet-size", &size_, 0);
	link_int("ns_cbr", "random", &random_, 0);
	running_ = 0;
}

void CBR_Agent::start()
{
	if (!running_) {
		running_ = 1;
		sendpkt();
		sched(interval_, 0);
	}
}

void CBR_Agent::stop()
{
	if (running_) {
		cancel(0);
		running_ = 0;
	}
}

void CBR_Agent::timeout(int)
{
	if (running_) {
		sendpkt();
		double t = interval_;
		if (random_)
			/* add some zero-mean white noise */
			t += interval_ * Random::uniform(-0.5, 0.5);
		sched(t, 0);
	}
}

void CBR_Agent::sendpkt()
{
	Packet* p = allocpkt();
	node_->transmit(p);
}

/*
 * $proc interval $interval
 * $proc size $size
 */
int CBR_Agent::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	if (argc == 2) {
		if (strcmp(argv[1], "start") == 0) {
			if (running_) {
				tcl.resultf("%s already running", name());
				return (TCL_ERROR);
			}
			start();
			return (TCL_OK);
		}
		if (strcmp(argv[1], "stop") == 0) {
			if (!running_) {
				tcl.resultf("%s already stopped", name());
				return (TCL_ERROR);
			}
			stop();
			return (TCL_OK);
		}
	}
	return (Agent::command(argc, argv));
}
