/*
 * Copyright (c) 1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * miscellaneous "ns" commands
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: misc.cc,v 1.5 96/01/26 14:33:40 mccanne Exp $ (LBL)";
#endif

#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include "scheduler.h"
#include "Tcl.h"
#include "random.h"

class AtEvent : public Event {
public:
	char* proc_;
};

class AtHandler : public Handler {
public:
	void handle(Event* event);
} at_handler;

void AtHandler::handle(Event* e)
{
	AtEvent* at = (AtEvent*)e;
	Tcl::instance().eval(at->proc_);
	delete at->proc_;
	delete at;
}

static class AtCommand : public TclObject {
public:
	AtCommand() : TclObject("ns-at") { }
        virtual int command(int argc, const char*const* argv);
} at_cmd;

extern double time_atof(const char*);

int AtCommand::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	if (argc == 2) {
		Scheduler& s = Scheduler::instance();
		Event* p = s.lookup(atoi(argv[1]));
		if (p != 0) {
			/*XXX make sure it really is an atevent*/
			s.cancel(p);
			AtEvent* ae = (AtEvent*)p;
			delete ae->proc_;
			delete ae;
		}
		return (TCL_OK);
	} else if (argc == 3) {
		double t = time_atof(argv[1]);
		const char* proc = argv[2];
	
		AtEvent* e = new AtEvent;
		int n = strlen(proc);
		e->proc_ = new char[n + 1];
		strcpy(e->proc_, proc);
		Scheduler& s = Scheduler::instance();
		double delay = t - s.clock();
		if (delay < 0) {
			tcl.result("ns-at: can't schedule command in past");
			return (TCL_ERROR);
		}
		s.schedule(&at_handler, e, delay);
		sprintf(tcl.buffer(), "%u", e->uid_);
		tcl.result(tcl.buffer());
		return (TCL_OK);
	}
	return (TclObject::command(argc, argv));
}

static class NowCommand : public TclObject {
public:
	NowCommand() : TclObject("ns-now") { }
        virtual int command(int argc, const char*const* argv);
} now_cmd;

int NowCommand::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	Scheduler& s = Scheduler::instance();
	sprintf(tcl.buffer(), "%.17g", s.clock());
	tcl.result(tcl.buffer());
	return (TCL_OK);
}

static class RunCommand : public TclObject {
public:
	RunCommand() : TclObject("ns-run") { }
        virtual int command(int argc, const char*const* argv);
} run_cmd;

int RunCommand::command(int argc, const char*const* argv)
{
	/* set global to 0 before calling object reset methods */
	Scheduler& s = Scheduler::instance();
	s.reset();
	TclObject::reset_all();
	s.run();
	return (TCL_OK);
}

static class RandomCommand : public TclObject {
public:
	RandomCommand() : TclObject("ns-random") { }
        virtual int command(int argc, const char*const* argv);
} random_cmd;

/*
 * ns-random
 * ns-random $seed 
 */
int RandomCommand::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	if (argc == 1) {
		sprintf(tcl.buffer(), "%u", Random::random());
		tcl.result(tcl.buffer());
	} else if (argc == 2) {
		int seed = atoi(argv[1]);
		if (seed == 0)
			seed = Random::seed_heuristically();
		else
			Random::seed(seed);
		tcl.resultf("%d", seed);
	}
	return (TCL_OK);
}

extern "C" char version_string[];

static class VersionCommand : public TclObject {
public:
	VersionCommand() : TclObject("ns-version") { }
        virtual int command(int argc, const char*const* argv) {
		Tcl::instance().result(version_string);
		return (TCL_OK);
	}
} version_cmd;
